==> pygar_client-0.4.1-py3-none-any.whl: gar/trsgar.py <==
import json
import asyncio
import argparse
import logging
import threading
import getpass
import sys
from gar import GARClient


async def main(main_args):
    """Sample GAR client to demonstrate subscribe and publish"""
    log_level = getattr(logging, main_args.log_level.upper())
    logging.basicConfig(level=log_level, format='%(asctime)s %(levelname)s %(filename)s:%(lineno)d – %(message)s')

    endpoint = main_args.url
    username = main_args.user if main_args.user else getpass.getuser()

    gar_client = GARClient(endpoint, username, allow_self_signed_certificate=main_args.allow_self_signed_certificate)
    client_thread = threading.Thread(target=gar_client.start)
    client_thread.daemon = True

    def on_heartbeat_timeout():
        logging.warning("Heartbeat timeout detected. Exiting application.")

    gar_client.register_heartbeat_timeout_handler(on_heartbeat_timeout)

    def on_gar_stopped():
        logging.info("GAR client stopped, exiting application.")

    gar_client.register_stopped_handler(on_gar_stopped)

    # pylint: disable=unused-argument
    def on_gar_error(error_message):
        logging.info("Exiting on error.")

    gar_client.register_error_handler(on_gar_error)

    heartbeat_received = threading.Event()
    def on_heartbeat(ms):
        logging.info("First heartbeat received %dms", ms)
        heartbeat_received.set()
        gar_client.clear_heartbeat_handler()

    gar_client.register_heartbeat_handler(on_heartbeat)

    client_thread.start()

    logging.info("Waiting for first heartbeat...")

    if not heartbeat_received.wait(timeout=29):  # seconds
        logging.error("Timed out waiting for first heartbeat")
        sys.exit(1)

    logging.info("Heartbeat received, starting stream.")

    try:
        if main_args.publish:
            key, topic, json_value = main_args.publish
            try:
                parsed_value = json.loads(json_value)  # Renamed from 'value'
                gar_client.publish_record(key, topic, parsed_value)
                # await asyncio.sleep(1) #TO DO - flush writes cleanly
                gar_client.logoff()
            except json.JSONDecodeError as json_exc:  # Renamed from 'e'
                logging.error("Invalid JSON value %s: %s", json_value, json_exc)
                gar_client.logoff()
        elif main_args.send_shutdown:
            gar_client.publish_shutdown()
            gar_client.logoff()
        else:
            subscription_mode = "Streaming" if main_args.streaming else "Snapshot"
            gar_client.subscribe("S1", mode=subscription_mode,
                                 key_name=None, topic_name=None,
                                 class_filter=main_args.__dict__["class"],
                                 key_filter=main_args.key_filter,
                                 topic_filter=main_args.topic_filter)

            if subscription_mode == "Snapshot":
                snapshot_received = threading.Event()
                def on_subscription_status(name, status):
                    logging.info("Snapshot received for subscription: %s (%s)", name, status)
                    snapshot_received.set()
                    gar_client.clear_subscription_status_handler()

                gar_client.register_subscription_status_handler(on_subscription_status)
                if not snapshot_received.wait(timeout=60):  # seconds
                    logging.error("Timed out waiting for snapshot")
                    gar_client.exit_code = 1

                gar_client.logoff()
    except KeyboardInterrupt:
        logging.info("Received KeyboardInterrupt, shutting down")
    except Exception as e:
        logging.error("Error in main loop: %s", e)

    client_thread.join()

    logging.info("Exiting with code %d", gar_client.exit_code)
    sys.exit(gar_client.exit_code)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="GAR Protocol Client")
    parser.add_argument("--url", type=str, default="ws://127.0.0.1:6502",
                        help="GAR WebSocket URL (default: ws://127.0.0.1:6502)")
    parser.add_argument("--user", type=str, default=None,
                        help="Username (default: OS environment username)")
    parser.add_argument("--allow-self-signed-certificate", action="store_true",
                        help="Allow self-signed certificates (default: False)")
    parser.add_argument("--log-level", type=str, default="INFO",
                        choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
                        help="Logging level (default: INFO)")
    parser.add_argument("--streaming", action="store_true",
                        help="Stay running and stream updates. Otherwise exit after snapshot received.")
    parser.add_argument("--key-filter", type=str, default=None,
                        help="Key filter regex pattern")
    parser.add_argument("--class", type=str, default=None,
                        help="Class name")
    parser.add_argument("--topic-filter", type=str, default=None,
                        help="Topic filter regex pattern")
    parser.add_argument("--send-shutdown", action="store_true",
                        help="Shut down the server")
    parser.add_argument("--publish", nargs=3, metavar=('key', 'topic', 'value'),
                        help="Publish a record with key, topic, and JSON value (e.g., --publish mykey mytopic '{\"data\": 123}')")

    args = parser.parse_args()
    asyncio.run(main(args))

==> pygar_client-0.4.2-py3-none-any.whl: gar/trsgar.py <==
import json
import asyncio
import argparse
import logging
import threading
import getpass
import sys
from gar import GARClient


async def main(main_args):
    """Sample GAR client to demonstrate subscribe and publish"""
    log_level = getattr(logging, main_args.log_level.upper())
    logging.basicConfig(level=log_level, format='%(asctime)s %(levelname)s %(filename)s:%(lineno)d – %(message)s')

    endpoint = main_args.url
    username = main_args.user if main_args.user else getpass.getuser()

    gar_client = GARClient(endpoint, username, allow_self_signed_certificate=main_args.allow_self_signed_certificate)
    client_thread = threading.Thread(target=gar_client.start)
    client_thread.daemon = True

    def on_heartbeat_timeout():
        logging.warning("Heartbeat timeout detected. Exiting application.")

    gar_client.register_heartbeat_timeout_handler(on_heartbeat_timeout)

    def on_gar_stopped():
        logging.info("GAR client stopped, exiting application.")

    gar_client.register_stopped_handler(on_gar_stopped)

    # pylint: disable=unused-argument
    def on_gar_error(error_message):
        logging.info("Exiting on error.")

    gar_client.register_error_handler(on_gar_error)

    heartbeat_received = threading.Event()
    def on_heartbeat(ms):
        logging.info("First heartbeat received %dms", ms)
        heartbeat_received.set()
        gar_client.clear_heartbeat_handler()

    gar_client.register_heartbeat_handler(on_heartbeat)

    client_thread.start()

    logging.info("Waiting for first heartbeat...")

    if not heartbeat_received.wait(timeout=29):  # seconds
        logging.error("Timed out waiting for first heartbeat")
        sys.exit(1)

    logging.info("Heartbeat received, starting stream.")

    try:
        if main_args.publish:
            key, topic, json_value = main_args.publish
            try:
                parsed_value = json.loads(json_value)  # Renamed from 'value'
                gar_client.publish_record(key, topic, parsed_value)
                # await asyncio.sleep(1) #TO DO - flush writes cleanly
                gar_client.logoff()
            except json.JSONDecodeError as json_exc:  # Renamed from 'e'
                logging.error("Invalid JSON value %s: %s", json_value, json_exc)
                gar_client.logoff()
        elif main_args.send_shutdown:
            gar_client.publish_shutdown()
            gar_client.logoff()
        else:
            subscription_mode = "Streaming" if main_args.streaming else "Snapshot"
            gar_client.subscribe("S1", mode=subscription_mode,
                                 key_name=None, topic_name=None,
                                 class_filter=main_args.__dict__["class"],
                                 key_filter=main_args.key_filter,
                                 topic_filter=main_args.topic_filter,
                                 include_unknown_keys=main_args.include_unknown_keys
                                 )

            if subscription_mode == "Snapshot":
                snapshot_received = threading.Event()
                def on_subscription_status(name, status):
                    logging.info("Snapshot received for subscription: %s (%s)", name, status)
                    snapshot_received.set()
                    gar_client.clear_subscription_status_handler()

                gar_client.register_subscription_status_handler(on_subscription_status)
                if not snapshot_received.wait(timeout=60):  # seconds
                    logging.error("Timed out waiting for snapshot")
                    gar_client.exit_code = 1

                gar_client.logoff()
    except KeyboardInterrupt:
        logging.info("Received KeyboardInterrupt, shutting down")
    except Exception as e:
        logging.error("Error in main loop: %s", e)

    client_thread.join()

    logging.info("Exiting with code %d", gar_client.exit_code)
    sys.exit(gar_client.exit_code)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="GAR Protocol Client")
    parser.add_argument("--url", type=str, default="ws://127.0.0.1:6502",
                        help="GAR WebSocket URL (default: ws://127.0.0.1:6502)")
    parser.add_argument("--user", type=str, default=None,
                        help="Username (default: OS environment username)")
    parser.add_argument("--allow-self-signed-certificate", action="store_true",
                        help="Allow self-signed certificates (default: False)")
    parser.add_argument("--log-level", type=str, default="INFO",
                        choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
                        help="Logging level (default: INFO)")
    parser.add_argument("--streaming", action="store_true",
                        help="Stay running and stream updates. Otherwise exit after snapshot received.")
    parser.add_argument("--key-filter", type=str, default=None,
                        help="Key filter regex pattern")
    parser.add_argument("--class", type=str, default=None,
                        help="Class name")
    parser.add_argument("--include-unknown-keys", action="store_true",
                        help="Include unknown keys when filtering for class")
    parser.add_argument("--topic-filter", type=str, default=None,
                        help="Topic filter regex pattern")
    parser.add_argument("--send-shutdown", action="store_true",
                        help="Shut down the server")
    parser.add_argument("--publish", nargs=3, metavar=('key', 'topic', 'value'),
                        help="Publish a record with key, topic, and JSON value (e.g., --publish mykey mytopic '{\"data\": 123}')")

    args = parser.parse_args()
    asyncio.run(main(args))
